@@ -7,17 +7,21 @@ pgxnclient -- installation/loading commands implementation
 # This file is part of the PGXN client
 
 import os
+import re
+import sys
 import shutil
+import difflib
 import logging
 from subprocess import PIPE
 
-from pgxnclient import Label, SemVer
+from pgxnclient import SemVer
 from pgxnclient.i18n import _, N_
 from pgxnclient.utils import sha1
 from pgxnclient.errors import BadChecksum, PgxnClientException
 from pgxnclient.network import download
 from pgxnclient.commands import Command, WithDatabase, WithMake, WithPgConfig
 from pgxnclient.commands import WithSpec, WithSpecLocal, WithSudo
+from pgxnclient.utils.strings import Identifier
 
 logger = logging.getLogger('pgxnclient.commands')
 
@@ -45,7 +49,7 @@ class Download(WithSpec, Command):
             raise PgxnClientException(
                 "sha1 missing from the distribution meta")
 
-        fin = self.api.download(spec.name, SemVer(data['version']))
+        fin = self.api.download(data['name'], SemVer(data['version']))
         fn = self._get_local_file_name(fin.url)
         fn = download(fin, fn, rename=True)
         self.verify_checksum(fn, chk)
@@ -92,7 +96,7 @@ class InstallUninstall(WithMake, WithSpecLocal, Command):
     def _run(self, dir):
         spec = self.get_spec()
         if spec.is_dir():
-            pdir = spec.dirname
+            pdir = os.path.abspath(spec.dirname)
         elif spec.is_file():
             pdir = self.unpack(spec.filename, dir)
         else:   # download
@@ -111,7 +115,6 @@ class InstallUninstall(WithMake, WithSpecLocal, Command):
     def maybe_run_configure(self, dir):
         fn = os.path.join(dir, 'configure')
         logger.debug("checking '%s'", fn)
-        # TODO: probably not portable
         if not os.path.exists(fn):
             return
 
@@ -177,6 +180,20 @@ class LoadUnload(WithPgConfig, WithDatabase, WithSpecLocal, Command):
     """
     Base class to implement the ``load`` and ``unload`` commands.
     """
+    @classmethod
+    def customize_parser(self, parser, subparsers, **kwargs):
+        subp = super(LoadUnload, self).customize_parser(
+            parser, subparsers, **kwargs)
+
+        subp.add_argument('--schema', metavar="SCHEMA",
+            type=Identifier.parse_arg,
+            help=_("use SCHEMA instead of the default schema"))
+
+        subp.add_argument('extensions', metavar='EXT', nargs='*',
+            help = _("only specified extensions [default: all]"))
+
+        return subp
+
     def get_pg_version(self):
         """Return the version of the selected database."""
         data = self.call_psql('SELECT version();')
@@ -185,7 +202,6 @@ class LoadUnload(WithPgConfig, WithDatabase, WithSpecLocal, Command):
         return pgver
 
     def parse_pg_version(self, data):
-        import re
         m = re.match(r'\S+\s+(\d+)\.(\d+)(?:\.(\d+))?', data)
         if m is None:
             raise PgxnClientException(
@@ -220,10 +236,16 @@ class LoadUnload(WithPgConfig, WithDatabase, WithSpecLocal, Command):
         cmdline.extend(self.get_psql_options())
         # load via pipe to enable psql commands in the file
         if not data:
+            logger.debug("loading sql from %s", filename)
             fin = open(filename, 'r')
             p = self.popen(cmdline, stdin=fin)
             p.communicate()
         else:
+            if len(data) > 105:
+                tdata = data[:100] + "..."
+            else:
+                tdata = data
+            logger.debug('running sql command: "%s"', tdata)
             p = self.popen(cmdline, stdin=PIPE)
             p.communicate(data)
 
@@ -261,6 +283,45 @@ class LoadUnload(WithPgConfig, WithDatabase, WithSpecLocal, Command):
                 "cannot find sql file for extension '%s': '%s'"
                 % (name, sqlfile))
 
+    def patch_for_schema(self, fn):
+        """
+        Patch a sql file to set the schema where the commands are executed.
+
+        If no schema has been requested, return the data unchanged.
+        Else, ask for confirmation and return the data for a patched file.
+
+        The schema is only useful for PG < 9.1: for proper PG extensions there
+        is no need to patch the sql.
+        """
+        schema = self.opts.schema
+
+        f = open(fn)
+        try: data = f.read()
+        finally: f.close()
+
+        if not schema:
+            return data
+
+        self._check_schema_exists(schema)
+
+        re_path = re.compile(r'SET\s+search_path\s*(?:=|to)\s*([^;]+);', re.I)
+        m = re_path.search(data)
+        if m is None:
+            newdata = ("SET search_path = %s;\n\n" % schema) + data
+        else:
+            newdata = re_path.sub("SET search_path = %s;" % schema, data)
+
+        diff = ''.join(difflib.unified_diff(
+            [r + '\n' for r in data.splitlines()],
+            [r + '\n' for r in newdata.splitlines()],
+            fn, fn + ".schema"))
+        msg = _("""
+In order to operate in the schema %s, the following changes will be
+performed:\n\n%s\n\nDo you want to continue?""")
+        self.confirm(msg % (schema, diff))
+
+        return newdata
+
     def _register_loaded(self, fn):
         if not hasattr(self, '_loaded'):
             self._loaded = []
@@ -270,26 +331,70 @@ class LoadUnload(WithPgConfig, WithDatabase, WithSpecLocal, Command):
     def _is_loaded(self, fn):
         return hasattr(self, '_loaded') and fn in self._loaded
 
+    def _check_schema_exists(self, schema):
+        cmdline = [self.find_psql()]
+        cmdline.extend(self.get_psql_options())
+        cmdline.extend(['-c', 'SET search_path=%s' % schema])
+        p = self.popen(cmdline, stdin=PIPE, stdout=PIPE, stderr=PIPE)
+        p.communicate()
+        if p.returncode:
+            raise PgxnClientException(
+                "schema %s does not exist" % schema)
+
+    def _get_extensions(self):
+        """
+        Return a list of pairs (name, sql file) to be loaded/unloaded.
+
+        Items are in loading order.
+        """
+        spec = self.get_spec()
+        dist = self.get_meta(spec)
+
+        if 'provides' not in dist:
+            # No 'provides' specified: assume a single extension named
+            # after the distribution. This is automatically done by PGXN,
+            # but we should do ourselves to deal with local META files
+            # not mangled by the PGXN upload script yet.
+            name = dist['name']
+            for ext in self.opts.extensions:
+                if ext <> name:
+                    raise PgxnClientException(
+                        "can't find extension '%s' in the distribution '%s'"
+                            % (name, spec))
+
+            return [ (name, None) ]
+
+        rv = []
+
+        if not self.opts.extensions:
+            # All the extensions, in the order specified
+            if len(dist['provides']) > 1 and sys.version_info < (2, 5):
+                logger.warn(_("can't guarantee extensions load order "
+                    "with Python < 2.5"))
+            for name, data in dist['provides'].items():
+                rv.append((name, data.get('file')))
+        else:
+            # Only the specified extensions
+            for name in self.opts.extensions:
+                try:
+                    data = dist['provides'][name]
+                except KeyError:
+                    raise PgxnClientException(
+                        "can't find extension '%s' in the distribution '%s'"
+                            % (name, spec))
+                rv.append((name, data.get('file')))
+
+        return rv
+
 
 class Load(LoadUnload):
     name = 'load'
     description = N_("load a distribution's extensions into a database")
 
     def run(self):
-        spec = self.get_spec()
-        dist = self.get_meta(spec)
-
-        # TODO: probably unordered before Python 2.7 or something
-        if 'provides' in dist:
-            for name, data in dist['provides'].items():
-                sql = data.get('file')
-                self.load_ext(name, sql)
-        else:
-            # No 'provides' specified: assume a single extension named
-            # after the distribution. This is automatically done by PGXN,
-            # but we should do ourselves to deal with local META files
-            # not mangled by the PGXN upload script yet.
-            self.load_ext(dist['name'], None)
+        items = self._get_extensions()
+        for (name, sql) in items:
+            self.load_ext(name, sql)
 
     def load_ext(self, name, sqlfile):
         logger.debug(_("loading extension '%s' with file: %s"),
@@ -328,15 +433,23 @@ The extension '%s' doesn't specify a SQL file.
 Do you want to load it?""")
                 % (name, fn))
 
+        # TODO: is confirmation asked only once? Also, check for repetition
+        # in unload.
         if self._is_loaded(fn):
             logger.info(_("file %s already loaded"), fn)
         else:
-            self.load_sql(fn)
+            data = self.patch_for_schema(fn)
+            self.load_sql(data=data)
             self._register_loaded(fn)
 
     def create_extension(self, name):
-        # TODO: namespace etc.
-        cmd = "CREATE EXTENSION %s;" % Label(name)
+        name = Identifier(name)
+        schema = self.opts.schema
+        cmd = ["CREATE EXTENSION", name]
+        if schema:
+            cmd.extend(["SCHEMA", schema])
+
+        cmd = " ".join(cmd) + ';'
         self.load_sql(data=cmd)
 
 
@@ -345,21 +458,15 @@ class Unload(LoadUnload):
     description = N_("unload a distribution's extensions from a database")
 
     def run(self):
-        spec = self.get_spec()
-        dist = self.get_meta(spec)
+        items = self._get_extensions()
 
-        # TODO: ensure ordering
-        if 'provides' in dist:
-            provs = dist['provides'].items()
-            provs.reverse()
-            for name, data in provs:
-                sql = data.get('file')
-                self.load_ext(name, sql)
-        else:
-            # Assume a single extension in the distribution - see Install
-            self.load_ext(dist['name'], None)
+        if not self.opts.extensions:
+            items.reverse()
 
-    def load_ext(self, name, sqlfile):
+        for (name, sql) in items:
+            self.unload_ext(name, sql)
+
+    def unload_ext(self, name, sqlfile):
         logger.debug(_("unloading extension '%s' with file: %s"),
             name, sqlfile)
 
@@ -396,11 +503,11 @@ to load the file '%s'.
 Do you want to execute it?""")
                 % (name, fn))
 
-        self.load_sql(fn)
+        data = self.patch_for_schema(fn)
+        self.load_sql(data=data)
 
     def drop_extension(self, name):
-        # TODO: namespace etc.
         # TODO: cascade
-        cmd = "DROP EXTENSION %s;" % Label(name)
+        cmd = "DROP EXTENSION %s;" % Identifier(name)
         self.load_sql(data=cmd)
 
