@@ -6,6 +6,7 @@ import shutil
 from urllib import quote
 
 from pgxnclient.utils import b
+from pgxnclient.errors import PgxnClientException, ResourceNotFound
 from pgxnclient.tests import unittest
 from pgxnclient.tests.testutils import ifunlink, get_test_filename
 
@@ -20,12 +21,14 @@ class FakeFile(object):
 def fake_get_file(url, urlmap=None):
     if urlmap: url = urlmap.get(url, url)
     fn = get_test_filename(quote(url, safe=""))
+    if not os.path.exists(fn):
+        raise ResourceNotFound(fn)
     f = FakeFile(fn, 'rb')
     f.url = url
     return f
 
 
-class ListTestCase(unittest.TestCase):
+class InfoTestCase(unittest.TestCase):
     def _get_output(self, cmdline):
         @patch('sys.stdout')
         @patch('pgxnclient.api.get_file')
@@ -38,7 +41,7 @@ class ListTestCase(unittest.TestCase):
 
         return do()
 
-    def test_list(self):
+    def test_info(self):
         output = self._get_output(['info', '--versions', 'foobar'])
         self.assertEqual(output, b("""\
 foobar 0.43.2b1 testing
@@ -46,24 +49,40 @@ foobar 0.42.1 stable
 foobar 0.42.0 stable
 """))
 
-    def test_list_op(self):
+    def test_info_op(self):
         output = self._get_output(['info', '--versions', 'foobar>0.42.0'])
         self.assertEqual(output, b("""\
 foobar 0.43.2b1 testing
 foobar 0.42.1 stable
 """))
 
-    def test_list_empty(self):
+    def test_info_empty(self):
         output = self._get_output(['info', '--versions', 'foobar>=0.43.2'])
         self.assertEqual(output, b(""))
 
+    def test_info_case_insensitive(self):
+        output = self._get_output(['info', '--versions', 'Foobar'])
+        self.assertEqual(output, b("""\
+foobar 0.43.2b1 testing
+foobar 0.42.1 stable
+foobar 0.42.0 stable
+"""))
+
+
+class CommandTestCase(unittest.TestCase):
+    def test_popen_raises(self):
+        from pgxnclient.commands import Command
+        c = Command([])
+        self.assertRaises(PgxnClientException,
+            c.popen, "this-script-doesnt-exist")
+
 
 class DownloadTestCase(unittest.TestCase):
     @patch('pgxnclient.api.get_file')
     def test_download_latest(self, mock):
         mock.side_effect = fake_get_file
 
-        fn = 'foobar-0.42.1.pgz'
+        fn = 'foobar-0.42.1.zip'
         self.assert_(not os.path.exists(fn))
 
         from pgxnclient.cli import main
@@ -77,7 +96,7 @@ class DownloadTestCase(unittest.TestCase):
     def test_download_testing(self, mock):
         mock.side_effect = fake_get_file
 
-        fn = 'foobar-0.43.2b1.pgz'
+        fn = 'foobar-0.43.2b1.zip'
         self.assert_(not os.path.exists(fn))
 
         from pgxnclient.cli import main
@@ -87,13 +106,27 @@ class DownloadTestCase(unittest.TestCase):
         finally:
             ifunlink(fn)
 
+    @patch('pgxnclient.api.get_file')
+    def test_download_ext(self, mock):
+        mock.side_effect = fake_get_file
+
+        fn = 'pg_amqp-0.3.0.zip'
+        self.assert_(not os.path.exists(fn))
+
+        from pgxnclient.cli import main
+        try:
+            main(['download', 'amqp'])
+            self.assert_(os.path.exists(fn))
+        finally:
+            ifunlink(fn)
+
     @patch('pgxnclient.api.get_file')
     def test_download_rename(self, mock):
         mock.side_effect = fake_get_file
 
-        fn = 'foobar-0.42.1.pgz'
-        fn1= 'foobar-0.42.1-1.pgz'
-        fn2= 'foobar-0.42.1-2.pgz'
+        fn = 'foobar-0.42.1.zip'
+        fn1= 'foobar-0.42.1-1.zip'
+        fn2= 'foobar-0.42.1-2.zip'
 
         for tmp in (fn, fn1, fn2):
             self.assert_(not os.path.exists(tmp))
@@ -129,7 +162,7 @@ class DownloadTestCase(unittest.TestCase):
 
         mock.side_effect = fakefake
 
-        fn = 'foobar-0.42.1.pgz'
+        fn = 'foobar-0.42.1.zip'
         self.assert_(not os.path.exists(fn))
 
         try:
@@ -143,6 +176,26 @@ class DownloadTestCase(unittest.TestCase):
         finally:
             ifunlink(fn)
 
+    @patch('pgxnclient.api.get_file')
+    def test_download_case_insensitive(self, mock):
+        mock.side_effect = fake_get_file
+
+        fn = 'pyrseas-0.4.1.zip'
+        self.assert_(not os.path.exists(fn))
+
+        from pgxnclient.cli import main
+        try:
+            main(['download', 'pyrseas'])
+            self.assert_(os.path.exists(fn))
+        finally:
+            ifunlink(fn)
+
+        try:
+            main(['download', 'Pyrseas'])
+            self.assert_(os.path.exists(fn))
+        finally:
+            ifunlink(fn)
+
     def test_version(self):
         from pgxnclient import Spec
         from pgxnclient.commands.install import Download
@@ -236,8 +289,6 @@ class InstallTestCase(unittest.TestCase):
         pop.returncode = 1
 
         from pgxnclient.cli import main
-        from pgxnclient.errors import PgxnClientException
-
         self.assertRaises(PgxnClientException, main, ['install', 'foobar'])
 
         self.assertEquals(mock_popen.call_count, 1)
@@ -299,7 +350,7 @@ class InstallTestCase(unittest.TestCase):
         mock_unpack.side_effect = unpack
 
         from pgxnclient.cli import main
-        main(['install', get_test_filename('foobar-0.42.1.pgz')])
+        main(['install', get_test_filename('foobar-0.42.1.zip')])
 
         self.assertEquals(mock_popen.call_count, 2)
         self.assertEquals(['make'], mock_popen.call_args_list[0][0][0][:1])
@@ -309,7 +360,7 @@ class InstallTestCase(unittest.TestCase):
 
         self.assertEquals(mock_unpack.call_count, 1)
         zipname, tmpdir = mock_unpack.call_args[0]
-        self.assertEqual(zipname, get_test_filename('foobar-0.42.1.pgz'))
+        self.assertEqual(zipname, get_test_filename('foobar-0.42.1.zip'))
         self.assertEqual(make_cwd, os.path.join(tmpdir, 'foobar-0.42.1'))
 
     @patch('pgxnclient.commands.Popen')
@@ -322,7 +373,7 @@ class InstallTestCase(unittest.TestCase):
         tdir = tempfile.mkdtemp()
         try:
             from pgxnclient.utils.zip import unpack
-            dir = unpack(get_test_filename('foobar-0.42.1.pgz'), tdir)
+            dir = unpack(get_test_filename('foobar-0.42.1.zip'), tdir)
 
             from pgxnclient.cli import main
             main(['install', dir])
@@ -360,7 +411,6 @@ class CheckTestCase(unittest.TestCase):
         pop.returncode = 1
 
         from pgxnclient.cli import main
-        from pgxnclient.errors import PgxnClientException
 
         self.assertRaises(PgxnClientException, main, ['check', 'foobar'])
 
@@ -387,7 +437,6 @@ class CheckTestCase(unittest.TestCase):
             "Please remove temp file 'regression.diffs' from current dir")
 
         from pgxnclient.cli import main
-        from pgxnclient.errors import PgxnClientException
 
         try:
             self.assertRaises(PgxnClientException, main, ['check', 'foobar'])
@@ -443,19 +492,19 @@ class LoadTestCase(unittest.TestCase):
 
         from pgxnclient.cli import main
 
-        main(['--yes', 'load', '--dbname', 'dbdb', 'foobar'])
+        main(['load', '--yes', '--dbname', 'dbdb', 'foobar'])
         args = mock_popen.call_args[0][0]
         self.assertEqual('dbdb', args[args.index('--dbname') + 1])
 
-        main(['--yes', 'load', '-U', 'meme', 'foobar'])
+        main(['load', '--yes', '-U', 'meme', 'foobar'])
         args = mock_popen.call_args[0][0]
         self.assertEqual('meme', args[args.index('--username') + 1])
 
-        main(['--yes', 'load', '--port', '666', 'foobar'])
+        main(['load', '--yes', '--port', '666', 'foobar'])
         args = mock_popen.call_args[0][0]
         self.assertEqual('666', args[args.index('--port') + 1])
 
-        main(['--yes', 'load', '-h', 'somewhere', 'foobar'])
+        main(['load', '--yes', '-h', 'somewhere', 'foobar'])
         args = mock_popen.call_args[0][0]
         self.assertEqual('somewhere', args[args.index('--host') + 1])
 
@@ -475,7 +524,7 @@ class LoadTestCase(unittest.TestCase):
         mock_isext.return_value = True
 
         from pgxnclient.cli import main
-        main(['--yes', 'load', get_test_filename('foobar-0.42.1.pgz')])
+        main(['load', '--yes', get_test_filename('foobar-0.42.1.zip')])
 
         self.assertEquals(mock_unpack.call_count, 0)
         self.assertEquals(mock_popen.call_count, 1)
@@ -483,7 +532,6 @@ class LoadTestCase(unittest.TestCase):
         self.assertEquals(pop.communicate.call_args[0][0],
             'CREATE EXTENSION foobar;')
 
-
     @patch('pgxnclient.commands.install.Load.is_extension')
     @patch('pgxnclient.commands.install.Load.get_pg_version')
     @patch('pgxnclient.commands.Popen')
@@ -499,10 +547,10 @@ class LoadTestCase(unittest.TestCase):
         tdir = tempfile.mkdtemp()
         try:
             from pgxnclient.utils.zip import unpack
-            dir = unpack(get_test_filename('foobar-0.42.1.pgz'), tdir)
+            dir = unpack(get_test_filename('foobar-0.42.1.zip'), tdir)
 
             from pgxnclient.cli import main
-            main(['--yes', 'load', dir])
+            main(['load', '--yes', dir])
 
         finally:
             shutil.rmtree(tdir)
@@ -512,3 +560,195 @@ class LoadTestCase(unittest.TestCase):
         self.assertEquals(pop.communicate.call_args[0][0],
             'CREATE EXTENSION foobar;')
 
+    @patch('pgxnclient.commands.install.Load.is_extension')
+    @patch('pgxnclient.commands.install.Load.get_pg_version')
+    @patch('pgxnclient.commands.Popen')
+    def test_load_extensions_order(self, mock_popen, mock_pgver, mock_isext):
+        pop = mock_popen.return_value
+        pop.returncode = 0
+        mock_pgver.return_value = (9,1,0)
+        mock_isext.return_value = True
+
+        tdir = tempfile.mkdtemp()
+        try:
+            from pgxnclient.utils.zip import unpack
+            dir = unpack(get_test_filename('foobar-0.42.1.zip'), tdir)
+            shutil.copyfile(
+                get_test_filename('META-manyext.json'),
+                os.path.join(dir, 'META.json'))
+
+            from pgxnclient.cli import main
+            main(['load', '--yes', dir])
+
+        finally:
+            shutil.rmtree(tdir)
+
+        self.assertEquals(mock_popen.call_count, 4)
+        self.assert_('psql' in mock_popen.call_args[0][0][0])
+        self.assertEquals(pop.communicate.call_args_list[0][0][0],
+            'CREATE EXTENSION foo;')
+        self.assertEquals(pop.communicate.call_args_list[1][0][0],
+            'CREATE EXTENSION bar;')
+        self.assertEquals(pop.communicate.call_args_list[2][0][0],
+            'CREATE EXTENSION baz;')
+        self.assertEquals(pop.communicate.call_args_list[3][0][0],
+            'CREATE EXTENSION qux;')
+
+    @patch('pgxnclient.commands.install.Unload.is_extension')
+    @patch('pgxnclient.commands.install.Unload.get_pg_version')
+    @patch('pgxnclient.commands.Popen')
+    def test_unload_extensions_order(self, mock_popen, mock_pgver, mock_isext):
+        pop = mock_popen.return_value
+        pop.returncode = 0
+        mock_pgver.return_value = (9,1,0)
+        mock_isext.return_value = True
+
+        tdir = tempfile.mkdtemp()
+        try:
+            from pgxnclient.utils.zip import unpack
+            dir = unpack(get_test_filename('foobar-0.42.1.zip'), tdir)
+            shutil.copyfile(
+                get_test_filename('META-manyext.json'),
+                os.path.join(dir, 'META.json'))
+
+            from pgxnclient.cli import main
+            main(['unload', '--yes', dir])
+
+        finally:
+            shutil.rmtree(tdir)
+
+        self.assertEquals(mock_popen.call_count, 4)
+        self.assert_('psql' in mock_popen.call_args[0][0][0])
+        self.assertEquals(pop.communicate.call_args_list[0][0][0],
+            'DROP EXTENSION qux;')
+        self.assertEquals(pop.communicate.call_args_list[1][0][0],
+            'DROP EXTENSION baz;')
+        self.assertEquals(pop.communicate.call_args_list[2][0][0],
+            'DROP EXTENSION bar;')
+        self.assertEquals(pop.communicate.call_args_list[3][0][0],
+            'DROP EXTENSION foo;')
+
+    @patch('pgxnclient.commands.install.Load.is_extension')
+    @patch('pgxnclient.commands.install.Load.get_pg_version')
+    @patch('pgxnclient.commands.Popen')
+    def test_load_list(self, mock_popen, mock_pgver, mock_isext):
+        pop = mock_popen.return_value
+        pop.returncode = 0
+        mock_pgver.return_value = (9,1,0)
+        mock_isext.return_value = True
+
+        tdir = tempfile.mkdtemp()
+        try:
+            from pgxnclient.utils.zip import unpack
+            dir = unpack(get_test_filename('foobar-0.42.1.zip'), tdir)
+            shutil.copyfile(
+                get_test_filename('META-manyext.json'),
+                os.path.join(dir, 'META.json'))
+
+            from pgxnclient.cli import main
+            main(['load', '--yes', dir, 'baz', 'foo'])
+
+        finally:
+            shutil.rmtree(tdir)
+
+        self.assertEquals(mock_popen.call_count, 2)
+        self.assert_('psql' in mock_popen.call_args[0][0][0])
+        self.assertEquals(pop.communicate.call_args_list[0][0][0],
+            'CREATE EXTENSION baz;')
+        self.assertEquals(pop.communicate.call_args_list[1][0][0],
+            'CREATE EXTENSION foo;')
+
+    @patch('pgxnclient.commands.install.Unload.is_extension')
+    @patch('pgxnclient.commands.install.Unload.get_pg_version')
+    @patch('pgxnclient.commands.Popen')
+    def test_unload_list(self, mock_popen, mock_pgver, mock_isext):
+        pop = mock_popen.return_value
+        pop.returncode = 0
+        mock_pgver.return_value = (9,1,0)
+        mock_isext.return_value = True
+
+        tdir = tempfile.mkdtemp()
+        try:
+            from pgxnclient.utils.zip import unpack
+            dir = unpack(get_test_filename('foobar-0.42.1.zip'), tdir)
+            shutil.copyfile(
+                get_test_filename('META-manyext.json'),
+                os.path.join(dir, 'META.json'))
+
+            from pgxnclient.cli import main
+            main(['unload', '--yes', dir, 'baz', 'foo'])
+
+        finally:
+            shutil.rmtree(tdir)
+
+        self.assertEquals(mock_popen.call_count, 2)
+        self.assert_('psql' in mock_popen.call_args[0][0][0])
+        self.assertEquals(pop.communicate.call_args_list[0][0][0],
+            'DROP EXTENSION baz;')
+        self.assertEquals(pop.communicate.call_args_list[1][0][0],
+            'DROP EXTENSION foo;')
+
+    @patch('pgxnclient.commands.install.Load.is_extension')
+    @patch('pgxnclient.commands.install.Load.get_pg_version')
+    @patch('pgxnclient.commands.Popen')
+    def test_load_missing(self, mock_popen, mock_pgver, mock_isext):
+        pop = mock_popen.return_value
+        pop.returncode = 0
+        mock_pgver.return_value = (9,1,0)
+        mock_isext.return_value = True
+
+        tdir = tempfile.mkdtemp()
+        try:
+            from pgxnclient.utils.zip import unpack
+            dir = unpack(get_test_filename('foobar-0.42.1.zip'), tdir)
+            shutil.copyfile(
+                get_test_filename('META-manyext.json'),
+                os.path.join(dir, 'META.json'))
+
+            from pgxnclient.cli import main
+            self.assertRaises(PgxnClientException, main,
+                ['load', '--yes', dir, 'foo', 'ach'])
+
+        finally:
+            shutil.rmtree(tdir)
+
+        self.assertEquals(mock_popen.call_count, 0)
+
+    @patch('pgxnclient.commands.install.Unload.is_extension')
+    @patch('pgxnclient.commands.install.Unload.get_pg_version')
+    @patch('pgxnclient.commands.Popen')
+    def test_unload_missing(self, mock_popen, mock_pgver, mock_isext):
+        pop = mock_popen.return_value
+        pop.returncode = 0
+        mock_pgver.return_value = (9,1,0)
+        mock_isext.return_value = True
+
+        tdir = tempfile.mkdtemp()
+        try:
+            from pgxnclient.utils.zip import unpack
+            dir = unpack(get_test_filename('foobar-0.42.1.zip'), tdir)
+            shutil.copyfile(
+                get_test_filename('META-manyext.json'),
+                os.path.join(dir, 'META.json'))
+
+            from pgxnclient.cli import main
+            self.assertRaises(PgxnClientException, main,
+                ['unload', '--yes', dir, 'foo', 'ach'])
+
+        finally:
+            shutil.rmtree(tdir)
+
+        self.assertEquals(mock_popen.call_count, 0)
+
+
+class SearchTestCase(unittest.TestCase):
+    @patch('sys.stdout')
+    @patch('pgxnclient.api.get_file')
+    def test_search_quoting(self, mock_get, stdout):
+        mock_get.side_effect = fake_get_file
+        from pgxnclient.cli import main
+        main(['search', '--docs', 'foo bar', 'baz'])
+
+
+if __name__ == '__main__':
+    unittest.main()
